from .async_docs import AsyncScraper
from .sync_docs  import SyncScraper
from .utils import executor

__version__ = "0.1.0"
__author__  = "Tom the Bomb"
__license__ = "MIT"
__copyright__ = "Copyright 2021 Tom the Bomb"
